@@ -1,10 +1,10 @@
 /* Libvisual - The audio visualisation framework.
  * 
- * Copyright (C) 2004, 2005 Dennis Smit <ds@nerds-incorporated.org>
+ * Copyright (C) 2004, 2005, 2006 Dennis Smit <ds@nerds-incorporated.org>
  *
  * Authors: Dennis Smit <ds@nerds-incorporated.org>
  *
- * $Id:
+ * $Id: lv_plugin.c,v 1.80.2.1 2006/03/04 12:32:47 descender Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
@@ -21,14 +21,24 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <dlfcn.h>
-#include <dirent.h>
+#include <gettext.h>
 
 #include "lvconfig.h"
+
+#if defined(VISUAL_OS_WIN32)
+#include <windows.h>
+#else
+#include <dlfcn.h>
+#endif
+
+#include <dirent.h>
+
 #include "lv_libvisual.h"
 #include "lv_plugin.h"
 #include "lv_log.h"
@@ -88,11 +98,11 @@ static int plugin_ref_dtor (VisObject *object)
 		visual_mem_free (ref->file);
 
 	if (ref->usecount > 0)
-		visual_log (VISUAL_LOG_CRITICAL, "A plugin reference with %d instances has been destroyed.", ref->usecount);
+		visual_log (VISUAL_LOG_CRITICAL, _("A plugin reference with %d instances has been destroyed."), ref->usecount);
 
 	if (ref->info != NULL)
 		visual_object_unref (VISUAL_OBJECT (ref->info));
-	
+
 	ref->file = NULL;
 	ref->info = NULL;
 
@@ -103,10 +113,10 @@ static int plugin_environ_dtor (VisObject *object)
 {
 	VisPluginEnviron *enve = VISUAL_PLUGINENVIRON (object);
 
-	if (enve->environ != NULL)
-		visual_object_unref (enve->environ);
+	if (enve->environment != NULL)
+		visual_object_unref (enve->environment);
 
-	enve->environ = NULL;
+	enve->environment = NULL;
 
 	return VISUAL_OK;
 }
@@ -115,14 +125,14 @@ static int plugin_dtor (VisObject *object)
 {
 	VisPluginData *plugin = VISUAL_PLUGINDATA (object);
 
-	if (plugin->ref != NULL)	
+	if (plugin->ref != NULL)
 		visual_object_unref (VISUAL_OBJECT (plugin->ref));
 
 	if (plugin->params != NULL)
 		visual_object_unref (VISUAL_OBJECT (plugin->params));
 
-	visual_list_destroy_elements (&plugin->environ);	
-	
+	visual_collection_destroy (VISUAL_COLLECTION (&plugin->environment));
+
 	plugin->ref = NULL;
 	plugin->params = NULL;
 
@@ -145,11 +155,11 @@ static char *get_delim_node (const char *str, char delim, int index)
 			/* Last section doesn't contain a delim */
 			if (end == NULL)
 				end = str + strlen (str);
-			
+
 			/* Cut off the delim that is in front */
 			if (i > 0)
 				start++;
-			
+
 			break;
 		}
 
@@ -331,10 +341,6 @@ VisRandomContext *visual_plugin_get_random_context (VisPluginData *plugin)
 /**
  * Retrieves the plugin specific part of a plugin.
  *
- * @see VISUAL_PLUGIN_ACTOR
- * @see VISUAL_PLUGIN_INPUT
- * @see VISUAL_PLUGIN_MORPH
- * 
  * @param plugin The pointer to the VisPluginData from which we want the plugin specific part.
  *
  * @return Void * pointing to the plugin specific part which can be cast.
@@ -416,23 +422,31 @@ VisList *visual_plugin_registry_filter (VisList *pluglist, const char *domain)
 	VisList *list;
 	VisListEntry *entry = NULL;
 	VisPluginRef *ref;
+	int ret;
 
 	visual_log_return_val_if_fail (pluglist != NULL, NULL);
 
-	list = visual_list_new (visual_object_list_destroyer);
+	list = visual_list_new (visual_object_collection_destroyer);
 
 	if (list == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot create a new list");
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot create a new list"));
 
 		return NULL;
 	}
 
 	while ((ref = visual_list_next (pluglist, &entry)) != NULL) {
 		
-		if (visual_plugin_type_member_of (ref->info->type, domain)) {
-			visual_object_ref (VISUAL_OBJECT (ref));
-			
-			visual_list_add (list, ref);
+		if ((ret = visual_plugin_type_member_of (ref->info->type, domain))) {
+			if (ret == TRUE) {
+				visual_object_ref (VISUAL_OBJECT (ref));
+				
+				visual_list_add (list, ref);
+			} else if (ret != FALSE) {
+				visual_log (VISUAL_LOG_WARNING, "%s", visual_error_to_string (ret));
+			}
+		}
+		else if (ret != FALSE) { /* FIXME XXX TODO, patch frmo duilio check how this works */
+			visual_log (VISUAL_LOG_WARNING, "%s", visual_error_to_string (ret));
 		}
 	}
 
@@ -489,15 +503,15 @@ const char *visual_plugin_get_prev_by_name (VisList *list, const char *name)
 {
 	VisListEntry *entry = NULL;
 	VisPluginRef *ref, *pref = NULL;
-	
+
 	visual_log_return_val_if_fail (list != NULL, NULL);
 
 	if (name == NULL) {
-		ref = visual_list_get (list, visual_list_count (list) - 1);
-		
+		ref = visual_list_get (list, visual_collection_size (VISUAL_COLLECTION (list)) - 1);
+
 		if (ref == NULL)
 			return NULL;
-		
+
 		return ref->info->plugname;
 	}
 
@@ -518,12 +532,63 @@ const char *visual_plugin_get_prev_by_name (VisList *list, const char *name)
 static int plugin_add_dir_to_list (VisList *list, const char *dir)
 {
 	VisPluginRef **ref;
-	char temp[1024];
-	struct dirent **namelist;
-	int i, j, n, len;
+	char temp[FILENAME_MAX];
+	int i, j, n;
+	size_t len;
 	int cnt = 0;
 
-	n = scandir (dir, &namelist, 0, alphasort);
+#if defined(VISUAL_OS_WIN32)
+	BOOL fFinished;
+	HANDLE hList;
+	TCHAR szDir[MAX_PATH+1];
+	TCHAR szSubDir[MAX_PATH+1];
+	WIN32_FIND_DATA FileData;
+
+	snprintf (szDir, MAX_PATH, "%s\\*", dir);
+
+	hList = FindFirstFile (szDir, &FileData);
+	if (hList == INVALID_HANDLE_VALUE) {
+		FindClose (hList);
+
+		return 0;
+	}
+
+	visual_mem_set (temp, 0, sizeof (temp));
+
+	fFinished = FALSE;
+
+	while (!fFinished) {
+		ref = NULL;
+
+		if (!(FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
+
+			snprintf (temp, 1023, "%s\\%s", dir, FileData.cFileName);
+
+			len = strlen (temp);
+			if (len > 5 && (strncmp (&temp[len - 5], ".dll", 5) == 0))
+				ref = visual_plugin_get_references (temp, &cnt);
+
+			if (ref != NULL) {
+				for (j = 0; j < cnt; j++)
+					visual_list_add (list, ref[j]);
+
+				/* This is the pointer pointer pointer, not a ref itself */
+				visual_mem_free (ref);
+			}
+
+			if (!FindNextFile (hList, &FileData)) {
+				if (GetLastError () == ERROR_NO_MORE_FILES) {
+					fFinished = TRUE;
+				}
+			}
+		}
+	}
+
+	FindClose (hList);
+#else
+	struct dirent **namelist;
+
+	n = scandir (dir, &namelist, NULL, alphasort);
 
 	if (n < 0)
 		return -1;
@@ -532,19 +597,21 @@ static int plugin_add_dir_to_list (VisList *list, const char *dir)
 	visual_mem_free (namelist[0]);
 	visual_mem_free (namelist[1]);
 
+	visual_mem_set (temp, 0, sizeof (temp));
+
 	for (i = 2; i < n; i++) {
 		ref = NULL;
 
 		snprintf (temp, 1023, "%s/%s", dir, namelist[i]->d_name);
 
 		len = strlen (temp);
-		if (len > 3 && (strncmp (&temp[len - 3], ".so", 3)) == 0)
+		if (len > 3 && (strncmp (&temp[len - 3], ".so", 3) == 0))
 			ref = visual_plugin_get_references (temp, &cnt);
 
 		if (ref != NULL) {
-			for (j = 0; j < cnt; j++) 
+			for (j = 0; j < cnt; j++)
 				visual_list_add (list, ref[j]);
-		
+
 			/* This is the pointer pointer pointer, not a ref itself */
 			visual_mem_free (ref);
 		}
@@ -553,6 +620,7 @@ static int plugin_add_dir_to_list (VisList *list, const char *dir)
 	}
 
 	visual_mem_free (namelist);
+#endif
 
 	return 0;
 }
@@ -578,7 +646,7 @@ int visual_plugin_unload (VisPluginData *plugin)
 	if (plugin->handle == NULL) {
 		visual_object_unref (VISUAL_OBJECT (plugin));
 
-		visual_log (VISUAL_LOG_CRITICAL, "Tried unloading a plugin that never has been loaded.");
+		visual_log (VISUAL_LOG_CRITICAL, _("Tried unloading a plugin that never has been loaded."));
 
 		return -VISUAL_ERROR_PLUGIN_HANDLE_NULL;
 	}
@@ -592,7 +660,12 @@ int visual_plugin_unload (VisPluginData *plugin)
 	if (plugin->info != NULL)
 		visual_object_unref (VISUAL_OBJECT (plugin->info));
 
+#if defined(VISUAL_OS_WIN32)
+	FreeLibrary (plugin->handle);	
+#else
 	dlclose (plugin->handle);
+#endif
+
 	plugin->info = NULL;
 
 	if (ref != NULL) {
@@ -621,7 +694,11 @@ VisPluginData *visual_plugin_load (VisPluginRef *ref)
 	VisTime time_;
 	VisPluginInfo *pluginfo;
 	VisPluginGetInfoFunc get_plugin_info;
+#if defined(VISUAL_OS_WIN32)
+	HMODULE handle;
+#else /* !VISUAL_OS_WIN32 */
 	void *handle;
+#endif
 	int cnt;
 
 	visual_log_return_val_if_fail (ref != NULL, NULL);
@@ -629,26 +706,43 @@ VisPluginData *visual_plugin_load (VisPluginRef *ref)
 
 	/* Check if this plugin is reentrant */
 	if (ref->usecount > 0 && (ref->info->flags & VISUAL_PLUGIN_FLAG_NOT_REENTRANT)) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot load plugin %s, the plugin is already loaded and is not reentrant.",
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot load plugin %s, the plugin is already loaded and is not reentrant."),
 				ref->info->plugname);
 
 		return NULL;
 	}
 
+#if defined(VISUAL_OS_WIN32)
+	handle = LoadLibrary (ref->file);
+#else
 	handle = dlopen (ref->file, RTLD_LAZY);
+#endif
 
 	if (handle == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot load plugin: %s", dlerror ());
-
+#if defined(VISUAL_OS_WIN32)
+		visual_log (VISUAL_LOG_CRITICAL, "Cannot load plugin: win32 error code: %d", GetLastError ());
+#else
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot load plugin: %s"), dlerror ());
+#endif
 		return NULL;
 	}
 
+#if defined(VISUAL_OS_WIN32)
+	get_plugin_info = (VisPluginGetInfoFunc) GetProcAddress (handle, "get_plugin_info");
+#else
 	get_plugin_info = (VisPluginGetInfoFunc) dlsym (handle, "get_plugin_info");
-	
-	if (get_plugin_info == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot initialize plugin: %s", dlerror ());
+#endif
 
+	if (get_plugin_info == NULL) {
+#if defined(VISUAL_OS_WIN32)
+		visual_log (VISUAL_LOG_CRITICAL, "Cannot initialize plugin: win32 error code: %d", GetLastError ());
+
+		FreeLibrary (handle);
+#else
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot initialize plugin: %s"), dlerror ());
+	
 		dlclose (handle);
+#endif
 
 		return NULL;
 	}
@@ -656,10 +750,14 @@ VisPluginData *visual_plugin_load (VisPluginRef *ref)
 	pluginfo = VISUAL_PLUGININFO (get_plugin_info (&cnt));
 
 	if (pluginfo == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot get plugin info while loading.");
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot get plugin info while loading."));
 
+#if defined(VISUAL_OS_WIN32)
+		FreeLibrary (handle);
+#else
 		dlclose (handle);
-		
+#endif
+
 		return NULL;
 	}
 
@@ -673,7 +771,7 @@ VisPluginData *visual_plugin_load (VisPluginRef *ref)
 	plugin->realized = FALSE;
 	plugin->handle = handle;
 
-	/* Now the plugin is set up and ready to be realized, also random seed it's random context */
+	/* Now the plugin is set up and ready to be realized, also random seed its random context */
 	visual_time_get (&time_);
 	visual_random_context_set_seed (&plugin->random, time_.tv_usec);
 
@@ -721,25 +819,67 @@ VisPluginRef **visual_plugin_get_references (const char *pluginpath, int *count)
 	VisPluginInfo *dup_info;
 	const char *plug_name;
 	VisPluginGetInfoFunc get_plugin_info;
+	int *plugin_version;
+#if defined(VISUAL_OS_WIN32)
+	HMODULE handle;
+#else /* !VISUAL_OS_WIN32 */
 	void *handle;
+#endif
 	int cnt = 1, i;
 
 	visual_log_return_val_if_fail (pluginpath != NULL, NULL);
 
+#if defined(VISUAL_OS_WIN32)
+	handle = LoadLibrary (pluginpath);
+#else
 	handle = dlopen (pluginpath, RTLD_LAZY);
-	
+#endif
+
 	if (handle == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot load plugin: %s", dlerror ());
+#if defined(VISUAL_OS_WIN32)
+		visual_log (VISUAL_LOG_CRITICAL, "Cannot load plugin: win32 error code: %d", GetLastError());
+#else
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot load plugin: %s"), dlerror ());
+#endif
 
 		return NULL;
 	}
 
+#if defined(VISUAL_OS_WIN32)
+	plugin_version = (int *) GetProcAddress (handle, VISUAL_PLUGIN_VERSION_TAG);
+#else
+	plugin_version = (int *) dlsym (handle, VISUAL_PLUGIN_VERSION_TAG);
+#endif
+
+	if (plugin_version == NULL || *plugin_version != VISUAL_PLUGIN_API_VERSION) {
+		visual_log (VISUAL_LOG_CRITICAL, _("Plugin %s is not compatible with version %s of libvisual"),
+				pluginpath, visual_get_version ());
+
+#if defined(VISUAL_OS_WIN32)
+		FreeLibrary (handle);
+#else
+		dlclose (handle);
+#endif
+
+		return NULL;
+	}
+
+#if defined(VISUAL_OS_WIN32)
+	get_plugin_info = (VisPluginGetInfoFunc) GetProcAddress (handle, "get_plugin_info");
+#else
 	get_plugin_info = (VisPluginGetInfoFunc) dlsym (handle, "get_plugin_info");
+#endif
 
 	if (get_plugin_info == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot initialize plugin: %s", dlerror ());
+#if defined(VISUAL_OS_WIN32)
+		visual_log (VISUAL_LOG_CRITICAL, "Cannot initialize plugin: win32 error code: %d", GetLastError ());
+
+		FreeLibrary (handle);
+#else
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot initialize plugin: %s"), dlerror ());
 
 		dlclose (handle);
+#endif
 
 		return NULL;
 	}
@@ -747,36 +887,25 @@ VisPluginRef **visual_plugin_get_references (const char *pluginpath, int *count)
 	plug_info = VISUAL_PLUGININFO (get_plugin_info (&cnt));
 
 	if (plug_info == NULL) {
-		visual_log (VISUAL_LOG_CRITICAL, "Cannot get plugin info");
-
-		dlclose (handle);
-		
-		return NULL;
-	}
-
-	/* FIXME we do leak the object when it fails the sanity check, tho
-	 * it is not always safe (with VERY stuff laying around) to unref it. */
-	
-	/* Check for API and struct size */
-	if (plug_info[0].struct_size != sizeof (VisPluginInfo) ||
-			plug_info[0].api_version != VISUAL_PLUGIN_API_VERSION) {
-
-		visual_log (VISUAL_LOG_CRITICAL, "Plugin %s is not compatible with version %s of libvisual",
-				pluginpath, visual_get_version ());
+		visual_log (VISUAL_LOG_CRITICAL, _("Cannot get plugin info"));
 
+#if defined(VISUAL_OS_WIN32)
+		FreeLibrary (handle);
+#else
 		dlclose (handle);
+#endif
 
 		return NULL;
 	}
 
 	ref = visual_mem_new0 (VisPluginRef *, cnt);
-	
+
 	for (i = 0; i < cnt; i++) {
 		ref[i] = visual_plugin_ref_new ();
 
 		dup_info = visual_plugin_info_new ();
 		visual_plugin_info_copy (dup_info, &plug_info[i]);
-		
+
 		ref[i]->index = i;
 		ref[i]->info = dup_info;
 		ref[i]->file = strdup (pluginpath);
@@ -785,9 +914,13 @@ VisPluginRef **visual_plugin_get_references (const char *pluginpath, int *count)
 		visual_object_unref (VISUAL_OBJECT (&plug_info[i]));
 	}
 
+#if defined(VISUAL_OS_WIN32)
+	FreeLibrary (handle);
+#else
 	dlclose (handle);
-	
-	*count = cnt;	
+#endif
+
+	*count = cnt;
 
 	return ref;
 }
@@ -796,25 +929,26 @@ VisPluginRef **visual_plugin_get_references (const char *pluginpath, int *count)
  * Private function to create the complete plugin registry from a set of paths.
  *
  * @param paths A pointer list to a set of paths.
+ * @param ignore_non_existing A flag that can be set with TRUE or FALSE to ignore non existing dirs.
  *
  * @return A newly allocated VisList containing the plugin registry for the set of paths.
  */
-VisList *visual_plugin_get_list (const char **paths)
+VisList *visual_plugin_get_list (const char **paths, int ignore_non_existing)
 {
 	VisList *list;
 	int i = 0;
 
-	list = visual_list_new (visual_object_list_destroyer);
-	
+	list = visual_list_new (visual_object_collection_destroyer);
+
 	while (paths[i] != NULL) {
 		if (plugin_add_dir_to_list (list, paths[i]) < 0) {
-			visual_log (VISUAL_LOG_WARNING, "Failed to add the %s directory to the plugin registry",
-					paths[i]);
+			if (ignore_non_existing == FALSE)
+				visual_log (VISUAL_LOG_WARNING, _("Failed to add the %s directory to the plugin registry"), paths[i]);
 		}
 
 		i++;
 	}
-	
+
 	return list;
 }
 
@@ -855,20 +989,6 @@ int visual_plugin_get_api_version ()
 	return VISUAL_PLUGIN_API_VERSION;
 }
 
-/**
- * Retrieves the VisSongInfo from a VisActorPlugin.
- *
- * @param actplugin Pointer to the VisActorPlugin from which the VisSongInfo is requested.
- *
- * @return The requested VisSongInfo or NULL on failure.
- */
-VisSongInfo *visual_plugin_actor_get_songinfo (VisActorPlugin *actplugin)
-{
-	visual_log_return_val_if_fail (actplugin != NULL, NULL);
-
-	return &actplugin->songinfo;
-}
-
 /**
  * Get the domain part from a plugin type string.
  *
@@ -906,9 +1026,27 @@ const char *visual_plugin_type_get_package (const char *type)
  */
 const char *visual_plugin_type_get_type (const char *type)
 {
+	char *str;
+	char *flags;
+	char *typestr;
+
 	visual_log_return_val_if_fail (type != NULL, NULL);
 
-	return get_delim_node (type, ':', 2);
+	str = get_delim_node (type, ':', 2);
+
+	flags = strchr (str, '.');
+
+	if (flags != NULL) {
+		typestr = visual_mem_malloc0 (flags - str);
+
+		strncpy (typestr, str, flags - str);
+
+		visual_mem_free (str);
+
+		return typestr;
+	}
+
+	return str;
 }
 
 /**
@@ -951,22 +1089,39 @@ VisPluginTypeDepth visual_plugin_type_get_depth (const char *type)
  */
 int visual_plugin_type_member_of (const char *domain, const char *type)
 {
+	char *ndomain;
+	char *tmp;
 	char *comp1;
 	char *comp2;
 	int diff = 0;
 	int i = 0;
 
+	visual_log_return_val_if_fail (domain != NULL, -VISUAL_ERROR_NULL);
 	visual_log_return_val_if_fail (type != NULL, -VISUAL_ERROR_NULL);
 
-	while (i < visual_plugin_type_get_depth (domain)) {
-		comp1 = get_delim_node (domain, ':', i);
+	ndomain = visual_mem_malloc0 (strlen (domain) + 1);
+	tmp = strchr (domain, '.');
+
+	if (tmp != NULL)
+		strncpy (ndomain, domain, tmp - domain);
+	else
+		strcpy (ndomain, domain);
+
+	while (i < visual_plugin_type_get_depth (ndomain)) {
+		comp1 = get_delim_node (ndomain, ':', i);
 		comp2 = get_delim_node (type, ':', i);
 
-		if (comp1 == NULL)
-			return FALSE;
+		if (comp1 == NULL || comp2 == NULL) {
+			if (comp1 != NULL)
+				visual_mem_free (comp1);
+
+			if (comp2 != NULL)
+				visual_mem_free (comp2);
+
+			visual_mem_free (ndomain);
 
-		if (comp2 == NULL)
 			return FALSE;
+		}
 
 		if (strcmp (comp1, comp2) != 0)
 			diff++;
@@ -977,12 +1132,91 @@ int visual_plugin_type_member_of (const char *domain, const char *type)
 		i++;
 	}
 
+	visual_mem_free (ndomain);
+
 	if (diff > 0)
 		return FALSE;
 
 	return TRUE;
 }
 
+/**
+ * Retrieves the flags section from the plugin type string.
+ *
+ * @param type The type string, containing the plugin type and optional flags.
+ *
+ * return NULL if no flags are found, the flags between the '[' and ']' braces on succes.
+ *	for example when the plugin type string is "Libvisual:core:actor.[special|something]"
+ *	the returned flag string would be "special|something". Keep in mind that the string is
+ *	allocated and should be freed after it's not being used anylonger.
+ */
+const char *visual_plugin_type_get_flags (const char *type)
+{
+	char *flagsret;
+	char *flagstr;
+
+	visual_log_return_val_if_fail (type != NULL, NULL);
+
+	flagstr = strstr (type, ".[");
+
+	if (flagstr == NULL)
+		return NULL;
+
+	flagstr += 2; /* Skip the ".[" */
+	
+	flagsret = visual_mem_malloc0 (strlen (flagstr) - 1);
+
+	strncpy (flagsret, flagstr, strlen (flagstr) - 1);
+	flagsret[strlen (flagstr) - 1] = '\0';
+	
+	return flagsret;
+}
+
+/**
+ * Checks if a certain flag is found within a plugin type string.
+ *
+ * @param type The type string, containing the plugin type and optional flags.
+ * @param flag The flag string to check for within the type string.
+ *
+ * @return TRUE in found, FALSE if not found, -VISUAL_ERROR_NULL on failure.
+ */
+int visual_plugin_type_has_flag (const char *type, const char *flag)
+{
+	char *flags;
+	char *nflag, *s;
+	
+	visual_log_return_val_if_fail (type != NULL, -VISUAL_ERROR_NULL);
+	visual_log_return_val_if_fail (flag != NULL, -VISUAL_ERROR_NULL);
+
+	nflag = flags = (char *) visual_plugin_type_get_flags (type);
+
+	if (flags == NULL)
+		return FALSE;
+
+	do {
+		s = strchr (nflag, '|');
+		
+		if (s != NULL) {
+			if (strncmp (nflag, flag, s - nflag - 1) == 0) {
+				visual_mem_free (flags);
+
+				return TRUE;
+			}
+		} else {
+			if (strcmp (nflag, flag) == 0) {
+				visual_mem_free (flags);
+
+				return TRUE;
+			}
+		}
+
+	} while ((nflag = strchr (nflag, '|') + 1));
+
+	visual_mem_free (flags);
+
+	return FALSE;
+}
+
 /**
  * Creates a VisPluginEnviron structure.
  *
@@ -1001,8 +1235,8 @@ VisPluginEnviron *visual_plugin_environ_new (const char *type, VisObject *envobj
 	visual_object_initialize (VISUAL_OBJECT (enve), TRUE, plugin_environ_dtor);
 
 	enve->type = type;
-	enve->environ = envobj;
-	
+	enve->environment = envobj;
+
 	return enve;
 }
 
@@ -1022,8 +1256,8 @@ int visual_plugin_environ_add (VisPluginData *plugin, VisPluginEnviron *enve)
 	visual_log_return_val_if_fail (enve->type != NULL, -VISUAL_ERROR_NULL);
 
 	visual_plugin_environ_remove (plugin, enve->type);
-		
-	return visual_list_add (&plugin->environ, enve);
+
+	return visual_list_add (&plugin->environment, enve);
 }
 
 /**
@@ -1042,11 +1276,11 @@ int visual_plugin_environ_remove (VisPluginData *plugin, const char *type)
 	visual_log_return_val_if_fail (plugin != NULL, -VISUAL_ERROR_PLUGIN_NULL);
 	visual_log_return_val_if_fail (type != NULL, -VISUAL_ERROR_NULL);
 
-	while ((enve = visual_list_next (&plugin->environ, &le)) != NULL) {
-		
+	while ((enve = visual_list_next (&plugin->environment, &le)) != NULL) {
+
 		/* Remove from list */
 		if (strcmp (enve->type, type) == 0) {
-			visual_list_delete (&plugin->environ, &le);
+			visual_list_delete (&plugin->environment, &le);
 
 			visual_object_unref (VISUAL_OBJECT (enve));
 
@@ -1069,14 +1303,14 @@ VisObject *visual_plugin_environ_get (VisPluginData *plugin, const char *type)
 {
 	VisPluginEnviron *enve;
 	VisListEntry *le = NULL;
-	
+
 	visual_log_return_val_if_fail (plugin != NULL, NULL);
 	visual_log_return_val_if_fail (type != NULL, NULL);
 
-	while ((enve = visual_list_next (&plugin->environ, &le)) != NULL) {
-		
+	while ((enve = visual_list_next (&plugin->environment, &le)) != NULL) {
+
 		if (strcmp (enve->type, type) == 0)
-			return enve->environ;
+			return enve->environment;
 	}
 
 	return NULL;
